/*  
 * 
 * KaBLAM.m	-- A fast action, bomb dropping, bomb catching game.  This is the
 *             "controller" object.  It handles user preferences from the Xox
 *             panel.  It is responsible for the introduction screen, the demo,
 *             and creating the key actors.  Keyboard events are also caught
 *             and dispatched from here.
 * 
 * 
 * You may freely copy, distribute, and reuse the code in this example.
 * The author disclaims any warranty of any kind, expressed or implied, as to
 * its fitness for any particular use.
 *
 * Written by Mark G. Tacchi, mtacchi@next.com
 *
 * Wed Jul 27 1994
 */

#import "BackView.h"
#import "KaBLAM.h"
#import "KB_Player.h"
#import "KB_Bomber.h"
#import "KB_Bomb.h"
#import "KB_GameOver.h"
#import "KB_InfoBomber.h"
#import "KB_InfoBomberHand.h"
#import "KB_InfoText.h"
#import "KB_InfoBomb.h"
#import "ActorMgr.h"
#import "KaBLAMDefs.h"
#import "CacheManager.h"
#import "SoundMgr.h"

#define DEFAULT_MOUSE_SENSITIVITY	2.0
#define DEFAULT_PLAYER_INPUT		1		//1= mouse, 0= keyboard

#define DEMO_LEVEL				12
@implementation KaBLAM

- newGame
{
  KB_Player *myPlayer;
  
  score= 0;
  [[scoreCell setIntValue: 0] display];
  lives= 3;
  nextBonus= 1000;
  myPlayer= (KB_Player *)player;
  gameOver= NO;
	
  if ([KB_themeSongSnd status]==NX_SoundPlaying)
    [(Sound *)KB_themeSongSnd stop];
	
  return self;
} /*newGame*/



// invoked only by the actor manager
- _createLevel:(int)lev
{
  char		title[64];
  int		i;
  
  badGuyCount= playerCount= 0;
  
  if (lev==1) {		//handle intro page
    id myBomber;
	
    [self setDemoMode: YES];
    [cacheMgr tileUsing:[[KB_Player class] findImageNamed:"kb_backgroundIntro.tiff"]];
    sprintf(title,"KaBLAM!");
    [[mainView window] setTitle:title];
	
	for (i= 0; i<20; i++)
	  [actorMgr newActor:kb_infoBomb for:self tag:i];
	myBomber= [actorMgr newActor:kb_infoBomber for:self tag:NEUTRAL];
	infoBomberHand= [myBomber hand];
  }
  else {
    [cacheMgr tileUsing:[[KB_Player class] findImageNamed:"kb_background.tiff"]];
    [self createPlayer];
    bomber= [actorMgr newActor:kb_bomber for:self tag:NEUTRAL];  
    [self setPauseForNextLevel:NO];
	
    if (demoMode)
	  sprintf(title,"KaBLAM!  -  Demo");
	else 
	  sprintf(title,"KaBLAM!  -  Level %d", lev-1);
    [[mainView window] setTitle:title];
	
  }
  
 return self;
} /*_createLevel:*/



- createPlayer
{
  player= [actorMgr newActor:kb_player for:self tag:lives];
  
  return self;
} /*createPlayer*/



- didActivate:(Actor *)theActor {
  if (theActor->actorType == kb_player) {
	//if this is demo mode, play the theme song
    if (demoMode && [KB_themeSongSnd status]!=NX_SoundPlaying && [soundMgr isSoundEnabled])
	 [KB_themeSongSnd play];
  } /*endif*/
  
  return self;
} /*didActivate:*/


- didRetire:(Actor *)theActor {return self;}


- keyDown:(NXEvent *)theEvent
{
  if (level==1) {
    if (gameOver) [actorMgr requestLevel: 0];
    else if (theEvent->data.key.charCode == ' ') {
	  if (demoMode) demoMode= NO;
      [actorMgr requestLevel: level+1];
	}
	[infoBomberHand provoke];
  }
  else {
	switch(theEvent->data.key.charCode) {
      case 'z':		// move left
        [player moveLeft];
        break;
      case 'x':		// move right
        [player moveRight];
        break;
      case ' ':		// stop
        if (gameOver || demoMode) [actorMgr requestLevel: 0];
        else if (pauseForNextLevel && ![[bomber bombList] count])
          [actorMgr requestLevel: level+1];
        else [player moveStop];
        break;
	}
  }
  
  return self;
} /*keyDown:*/



- keyUp:(NXEvent *)theEvent
{
  switch(theEvent->data.key.charCode) {
    case 'z':		// left
      [player stopMoveLeft];
      break;
    case 'x':		// right
      [player stopMoveRight];
      break;
    case ' ':		//stop
      break;
  }
	
	return self;
} /*keyUp:*/



- infoView {return infoView;}
- scenarioSelected {return self;}
- scenarioDeselected {return self;}


- (BOOL) createCollisionListsFrom: (List *)theActors
	good: (List *)goodList
	bad: (List *)badList
	ugly: (List *)uglyList
{
	Actor *act;
	int i, count = [theActors count];
	for (i=0; i<count; i++)
	{
		act = (Actor *)[theActors objectAt:i];
		if ((act->x > gx + 1.25*xOffset) || (act->x < gx - 1.25*xOffset) || 
				(act->y > gy + 1.25*yOffset) || (act->y < gy - 1.25*yOffset) ||
				!act->employed)
			continue;
		switch((int)(act->alliance))
		{
			case GOOD:
				[goodList addObject:act];
				break;
			case EVIL:
				[badList addObject:act];
				break;
			case DESTROYALL:
				[uglyList addObject:act];
				break;
		}
	}
	return YES;
} /*createCollisionListsFrom:*/



- init
{
  char	path[256];
  int		i;
  char	buf[MAXPATHLEN+1];
  id cv, subviews;

  [super init];

  if ([[NXBundle bundleForClass:[self class]]
    getPath:path
    forResource:"KaBLAM"
    ofType:"nib"]) {
    [NXApp loadNibFile:path
    owner:self
    withNames:NO
    fromZone:[self zone]];
			
    cv= [uselessView contentView];
    subviews= [cv subviews];
    while ([subviews count] > 0)
      [scoreView addSubview:[subviews objectAt:0]];
  } /*endif*/

  kb_player= (int)[KB_Player class];
  kb_bomber= (int)[KB_Bomber class];
  kb_bomb= (int)[KB_Bomb class];  
  kb_gameOver= (int)[KB_GameOver class];  
  
  kb_infoBomber= (int)[KB_InfoBomber class];
  kb_infoBomberHand= (int)[KB_InfoBomberHand class];
  kb_infoText= (int)[KB_InfoText class];
  kb_infoBomb= (int)[KB_InfoBomb class];

  //add sounds
  if ([[NXBundle bundleForClass:[self class]]
    getPath:path
    forResource:"kb_kablamThemeSong"
    ofType:"snd"]) {
    KB_themeSongSnd= [[Sound alloc] initFromSoundfile:path];
  } /*endif*/

  kb_explodeSnd= [soundMgr addSound:"kb_explode" sender:self];
  for (i=0; i<8; i++) {
    sprintf(buf,"kb_catch%d",i);
    kb_catchSnd[i]= [soundMgr addSound: buf sender:self];
  } /*nexti*/
  
  kb_gameOverSnd= [soundMgr addSound:"kb_gameOver" sender:self];
  kb_extraGuySnd= [soundMgr addSound:"kb_extraGuy" sender:self];
  kb_slideWhistleSnd= [soundMgr addSound:"kb_slideWhistle" sender:self];

  [self adjustSettings:self];
	
  return self;
} /*init*/



- tile
{
  NXRect r;

  [gcontentView getBounds:&r];
  r.size.height -= KB_SCOREVIEW_HEIGHT;
  r.origin.y += KB_SCOREVIEW_HEIGHT;
  [abackView setFrame:&r];
  
  [gcontentView getBounds:&r];
  r.size.height = KB_SCOREVIEW_HEIGHT;
  r.origin.y= 0;
  [scoreView setFrame:&r];
  
  [gcontentView addSubview:abackView];
  [gcontentView addSubview:scoreView];
  
  return abackView;
} /*tile*/



- adjustSettings:sender
{
  
  char *defaultValue;

  //read defaults from defaults database
  if ((defaultValue= (char *) NXReadDefault([NXApp appName],"KB_MouseSensitivity"))!=NULL) {
    mouseSensitivity= (float)atof(defaultValue);
    [mouseSensitivitySlider setIntValue: mouseSensitivity];
  }
  else
    mouseSensitivity= DEFAULT_MOUSE_SENSITIVITY;
	
  if ((defaultValue= (char *) NXReadDefault([NXApp appName],"KB_PlayerInput"))!=NULL) {
    mouseAsInput= (int)atoi(defaultValue);
    [playerInputMatrix selectCellWithTag: !mouseAsInput];
  }
  else
    mouseAsInput= DEFAULT_PLAYER_INPUT;

  if ((defaultValue= (char *) NXReadDefault([NXApp appName],"KB_HighScore"))!=NULL) {
    [highScoreCell setStringValue: defaultValue];
    [highScoreCell display];
    highScore= atoi(defaultValue);
  }
  else
    highScore= 0;

  return self;
} /*adjustSettings:*/



- gotoLevel:sender
{
  return self;
} /*gotoLevel:*/



- (int) level {
  int currentLevel;
  
  if ([self demoMode]) currentLevel= DEMO_LEVEL;
  else currentLevel= level;
  
  return currentLevel;
} /*level*/



- (BOOL)newWindowContentSize:(NXSize *)s
{
	s->width= KB_SCREENWIDTH;
	s->height= KB_SCREENHEIGHT + KB_SCOREVIEW_HEIGHT;
	
	return YES;
} /*newWindowContentSize:*/



- (COLLISION_PARADIGM)collisionParadigm { return GOOD_V_EVIL; }

- (int) lives{return lives;}

- (BOOL) windowResizable:(NXSize *)fixedSize { return NO; }



- (float) lowestBombLocation
{
  id lowestBomb= [[bomber bombList] objectAt: 0];
  float lowestPosition;
  
  if (!lowestBomb) lowestPosition= 0;  //seek center location if no bombs
  else lowestPosition= [lowestBomb position];
  
  return lowestPosition;
} /*lowestBombLocation*/



- (int) score { return score; }

- setScore: (int) newScore
{
  score= newScore;
  [scoreCell setIntValue: newScore];
  return self;
} /*setScore:*/



- addScore: (int) moreScore
{
  if (demoMode) return self;		//don't modify score if in demo mode
  
  score+= moreScore;
  [scoreCell setIntValue: score];
  if (score >= nextBonus) {
    if (lives<3) {
      lives++;
	  [player setNumPails: lives];
      [soundMgr playSound: kb_extraGuySnd at:0.5];
	} /*endif*/
    nextBonus += 1000;
  } /*endif*/
  
  if (score>highScore) {
    highScore= score;
    [highScoreCell setIntValue: highScore];
  } /*endif*/
  
  return self;
} /*addScore:*/



- (BOOL) pauseForNextLevel { return pauseForNextLevel;}

- setPauseForNextLevel: (BOOL) pause
{
  if (!demoMode) pauseForNextLevel= pause;
  
  return self;
} /*setPauseForNextLevel:*/



- loseLife
{
  lives--;
  level-= 2;  if (level<1) level= 1;
  if (lives<=0) {  //GAME OVER
    char buf[64];
	
    gameOver= YES; 
	[actorMgr newActor:kb_gameOver for:self tag:NEUTRAL];
    sprintf(buf,"%d", highScore);  //write out high score
    NXWriteDefault([NXApp appName],"KB_HighScore", buf);
  } /*endif*/
  
  return self;
  
} /*loseLife*/



- (float) mouseSensitivity { return mouseSensitivity; }

- setMouseSensitivity: sender
{
  char buf[64];
  
  mouseSensitivity= [sender floatValue];
  if (player) [player setMouseScale: mouseSensitivity];
  sprintf(buf,"%g", mouseSensitivity);
  NXWriteDefault([NXApp appName],"KB_MouseSensitivity", buf);
  
  return self;
} /*setMouseSensitivity:*/



- (BOOL) playerInput { return mouseAsInput; }

- setPlayerInput: sender
{
  char buf[8];

  mouseAsInput= ![[sender selectedCell] tag];
  if (player) [player setMouseAsInput: mouseAsInput];
  sprintf(buf,"%d", mouseAsInput);
  NXWriteDefault([NXApp appName],"KB_PlayerInput", buf);
  
  return self;
} /*setPlayerInput:*/



- (BOOL) demoMode { return demoMode; }

- setDemoMode: (BOOL) mode
{
  demoMode= mode;
  
  return self;
} /*setDemoMode:*/
@end

